'*******************************************************************************
'    FILENAME        : header.vbs
'    DESCRIPTION     : This script will insert commented header information into
'                      the top of the current file with the filename, username,
'                      and dates automatically inserted. If the file is an ASP/PHP
'                      file then the Request Collections and any session variables
'                      or include files used on the page will also be automatically
'                      inserted. The header will be commented out in a style
'                      appropriate to the file type being edited. ASP/PHP exclusive
'                      fields will be returned sorted alphabetically.

'                      Subsequent calls to this script will update the "Last Updated",
'                      "Updated by" and any ASP/PHP exclusive fields, without modifying
'                      anything else.

'                      Supported file types:
'                      asa, asp, aspx, ascx, asmx, asa, asax, inc, bas, cfm, 
'                      cfml, css, htm, html, js, php, php3, php4, php5, ph3, 
'                      ph4, phtm, phtml, pl, shtml, sql, vbs, c, cs

'    CREATED         : 14 March 2006
'    CREATED BY      : James Swan

'    NOTE            : Configure personal settings beginning on LINE 59
'                      Since this script uses Regular Expressions to extract the
'                      document info, VBScript version 5 is needed to run it.
'                      This is the version of the VBScript scripting engine released
'                      with Internet Explorer 5.
'                      This script has only been tested on Windows 2000 and XP

'   You may distribute this script freely, but please keep this header intact.

'   Version History : 0.2 (2007-07-03) - added Application Variables to header
'                                      - added document URL field to header
'                                      - fixed the default comment style
'                     0.3 (2008-01-21) - fixed bug to extract database table names
'                                        that are wrapped in square brackets
'                     0.4 (2008-09-10) - changed spacing in header from real tabs
'                                        to spaces
'                     0.5 (2008-12-12) - added document's function and subroutines
'                                        to header.
'                                      - added ability to configure stored procedure prefix
'                                      - fixed minor bugs with URL field
'                                      - added check for C# in aspx files
'                     0.6 (2011-11-11) - fixed bug if login name not found in array
'   TODO            : Grab inserted line count for returning cursor to original position
'                     Grab function comments
'*******************************************************************************
    
    Option Explicit
    
    Const MODULE_NAME = "InsertHeader"
    Const MODULE_VER = "0.6"
    Const MODULE_TITLE = "Insert Info Header"
    
    Const INCORRECT = 0
    Const CORRECT = 1
    
    '**************************** CONFIGURE ************************************
    ' the name to be inserted after "Created by:" when the header is inserted in
    ' documents that have been created previously
    Const DEFAULT_AUTHOR = "My Name"
    
    ' when inserting "Created by:" or "Updated by" into a document, the users Windows
    ' login name is used... if that login name exists as an element in this array,
    ' it will be substituted by the value of the DEFAULT_AUTHOR constant above
    Dim arrLoginNames(0)
    arrLoginNames(0) = ""
    
    ' when inserting the URL into the header the http path/domain name cannot 
    ' always be retrieved programatically - add any anomolies to this array and 
    ' they will be corrected
    Dim arrHTTP_Paths(1, 0)
    
    arrHTTP_Paths(INCORRECT, 0) = "C:/webserver"
    arrHTTP_Paths(CORRECT,   0) = "localhost"
    
    Dim arrLocal_Paths(1, 0)
    
    arrLocal_Paths(INCORRECT, 0) = "C:\"
    arrLocal_Paths(CORRECT,   0) = "\\ServerName\"
    
    ' used by regular expressions to extract database object names from the document
    Const DB_TABLE_PREFIX = "tbl"
    Const STORED_PROCEDURE_PREFIX = "sp_"
    '***************************************************************************
    
    Dim strFileExt, strLineStart
    
    Sub Init()
        addMenuItem MODULE_TITLE, "Utilities", MODULE_NAME, "CTRL+ALT+I"
    End Sub
    
    Sub InsertHeader()
        Call DoInsert(True) 
    End Sub
    
    Sub DoInsert(blnFirstInsert)
        Dim objEditor, objRegExp, objMatches, objFSO, objFile, objWSHNetwork
        Dim objUser
        Dim strFileContents, strFilePath, strFileName, strNow, strCreated
        Dim strHeader, strProtocol, strHeaderStart, strHeaderEnd
        Dim strTopLine, strLanguage, strUserName, strTabs, strCreatedBy
        Dim strLoginName, strUserDomain, strAuthorName, strElement
        Dim reHeaderPattern, reLine
        Dim i, intXPos, intYPos, intQuotePos, intExtraLines, intDefaultLocale
        Dim datNow, datCreated
        Dim arrPrevious
        
        Set objEditor = newEditor()
        
        With objEditor
            .assignActiveEditor()
            strFileContents = .Text()
            strFilePath = .Filename()
            
            strFileName = Mid(strFilePath, InStrRev(strFilePath, "\") + 1)
            strFileExt = LCase(Mid(strFileName, InStrRev(strFileName, ".") + 1))
            
            ' store current cursor position.
            intXPos = .caretX()
            intYPos = .caretY()
            
            ' go to top line and grab text
            .setCaretPos 0, 1
            strTopLine = .lineText()
        End With
        
        strHeaderStart = "<%" & vbCrLf
        strHeaderEnd = "%>" & vbCrLf
        strLineStart = "'   "
        strTabs = ""        ' non ASP files require two tabs less in the header formatting
        intExtraLines = 13  ' non ASP files have five lines less in the header
            
        '-----------------------------------------------------------------------
        '   set appropriate comment style
        '-----------------------------------------------------------------------
        Select Case strFileExt
            Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "inc"
                strTabs = "        "
                intExtraLines = 20
            Case "vbs", "vb", "bas", "vba"
                strHeaderStart = "'*******************************************************************************" & vbCrLf
                strHeaderEnd = strHeaderStart
            Case "js", "css", "sql", "c", "cs"
                strHeaderStart = "/*******************************************************************************" & vbCrLf
                strHeaderEnd = "*******************************************************************************/    " & vbCrLf
                strLineStart = "    "
            Case "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                strHeaderStart = "<?php" & vbCrLf & "/*******************************************************************************" & vbCrLf
                strHeaderEnd = "*******************************************************************************/    " & vbCrLf & "?>" & vbCrLf
                strLineStart = "    "
                strTabs = "        "
                intExtraLines = 20
            Case "pl"
                strHeaderStart = "# *******************************************************************************" & vbCrLf
                strHeaderEnd = "#   *******************************************************************************" & vbCrLf
                strLineStart = "#   "
            Case Else
                strHeaderStart = "<!--" & vbCrLf
                strHeaderEnd = "-->" & vbCrLf
                strLineStart = "    "
        End Select
        
        Set objRegExp = New RegExp
        objRegExp.Global = True
        objRegExp.IgnoreCase = True
        
        '-----------------------------------------------------------------------
        '   check for ASP <%@ %> command or CFSETTING
        '-----------------------------------------------------------------------
        If strFileExt = "cfm" OR strFileExt = "cfml" Then
            
            objRegExp.Pattern = "(<CFSETTING *EnableCFOutputOnly)|(<CFSILENT[^>]*>)"
            Set objMatches = objRegExp.Execute(strTopLine)
            
            If objMatches.Count > 0 Then
                ' move cursor so as to insert header after command on first line
                objEditor.setCaretPos 0, 2
            End If
            
        ElseIf strFileExt = "asp" OR strFileExt = "aspx" OR strFileExt = "ascx" Then
            
            objRegExp.Pattern = "<% ?@"
            Set objMatches = objRegExp.Execute(strTopLine)
            
            If objMatches.Count > 0 Then
                
                ' move cursor so as to insert header after command on first line
                objEditor.setCaretPos 0, 2
                
                ' -- search for LANGUAGE type so as to reset appropriate comment style --
                objRegExp.Pattern = "LANGUAGE *= *""?(vbscript|javascript|jscript|perlscript|c#|vb)"
                Set objMatches = objRegExp.Execute(strTopLine)
                '---------------------------------------------------------------
                
                If objMatches.Count > 0 Then
                    
                    strLanguage = objMatches.Item(0)
                    
                    ' get string portion to the right of "="
                    strLanguage = Trim(LCase(Mid(strLanguage, InStr(strLanguage, "=") + 1)))
                    
                    ' ----- if quotes were used strip leading quote ------------
                    intQuotePos = InStr(strLanguage, """")
                    If Not intQuotePos = -1 Then
                        strLanguage = Trim(Mid(strLanguage, intQuotePos + 1))
                    End If
                    ' ----------------------------------------------------------
                    
                    Select Case strLanguage
                        Case "vbscript", "vb"
                        Case "perlscript"
                            strLineStart = "#   "
                        Case Else
                            strLineStart = "//  "
                    End Select
                    
                End If
                
            End If
            
        End If
        
        datNow = Now()
        
        '-----------------------------------------------------------------------
        '   get user name
        '-----------------------------------------------------------------------
        Set objWSHNetwork = CreateObject("WScript.Network")
            With objWSHNetwork
                strUserName   = .UserName
                strUserDomain = .UserDomain
            End With
        Set objWSHNetwork = Nothing
        
        '-----------------------------------------------------------------------
        '   get user's full name
        '-----------------------------------------------------------------------
        On Error Resume Next
            Set objUser = GetObject("WinNT://" & strUserDomain & "/" & strUserName)
                strAuthorName = objUser.FullName
            Set objUser = Nothing
        On Error GoTo 0
        
        If Len(strAuthorName) = 0 Then
            strAuthorName = strUserName
        End if
        
        '-----------------------------------------------------------------------
        '   convert user name to given name
        '-----------------------------------------------------------------------
        For i = 0 To UBound(arrLoginNames)
            strLoginName = arrLoginNames(i)
            
            If UCase(strUserName) = UCase(strLoginName) Then
                strAuthorName = DEFAULT_AUTHOR
                Exit For
            End If
        Next
        
        '-----------------------------------------------------------------------
        '   get file's creation date
        '-----------------------------------------------------------------------
        If Len(strFilePath) > 0 Then
            strProtocol = LCase(Left(strFilePath, 6))
            ' can only get the creation date on local files
            If strProtocol <> "ftp://" And strProtocol <> "rds://" Then
                Set objFSO = CreateObject("Scripting.FileSystemObject")
                
                If objFSO.FileExists(strFilePath) Then
                    Set objFile = objFSO.GetFile(strFilePath)
                    datCreated = CDate(objFile.DateCreated)
                    Set objFile = Nothing
                    strCreatedBy = DEFAULT_AUTHOR
                Else
                    datCreated = datNow
                    strCreatedBy = strAuthorName
                End If
                
                Set objFSO = Nothing
            End If
        Else
            datCreated = datNow
            strCreatedBy = strAuthorName
        End If
        
        intDefaultLocale = GetLocale
        SetLocale (2057)
        
        strNow = FormatDateTime(datNow, 1) & " (" & FormatDateTime(datNow, 3) & ")"
        strCreated = FormatDateTime(datCreated, 1) & " (" & FormatDateTime(datCreated, 3) & ")"
        
        SetLocale (intDefaultLocale)
        
        '-----------------------------------------------------------------------
        '   search for existing header
        '-----------------------------------------------------------------------
        reLine = ".*\n"
        
        reHeaderPattern = reHeaderPattern &_
        strLineStart & "Filename\s+:" & reLine &_
        strLineStart & "URL\s+:" & reLine & reLine &_
        strLineStart & "Created\s+:" & reLine &_
        strLineStart & "Created by\s+:" & reLine & reLine &_
        strLineStart & "Last Updated\s+:" & reLine &_
        strLineStart & "Updated by\s+:" & reLine & reLine & "[\s\S]*" &_
        strLineStart & "Comments\s+:"
        
        objRegExp.Pattern = reHeaderPattern
        Set objMatches = objRegExp.Execute(strFileContents)
        
        '-----------------------------------------------------------------------
        '   create/update header
        '-----------------------------------------------------------------------
        If objMatches.Count > 0 Then
        
            Dim intCount, strItem
            arrPrevious = Split(objMatches.Item(0), ": ")
            
            ' trim spaces and line break from relevant elements
            For intCount = 2 To 5
                strItem = arrPrevious(intCount)
                strItem = Trim(Left(strItem, InStr(strItem, vbCrLf) - 1))
                arrPrevious(intCount) = strItem
            Next
            
            Select Case strFileExt
                Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                    strHeader = strHeader & ShowElement("Filename                : ", strFileName, vbCrLf)
                    strHeader = strHeader & ShowElement("URL                     : ", HTTP_Path(strFilePath), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Created                 : ", arrPrevious(3), vbCrLf)
                    strHeader = strHeader & ShowElement("Created by              : ", arrPrevious(4), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Last Updated            : ", strNow, vbCrLf)
                    strHeader = strHeader & ShowElement("Updated by              : ", strAuthorName, vbCrLf & vbCrLf)
                Case Else
                    strHeader = strHeader & ShowElement("Filename        : ", strFileName, vbCrLf)
                    strHeader = strHeader & ShowElement("URL             : ", HTTP_Path(strFilePath), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Created         : ", arrPrevious(3), vbCrLf)
                    strHeader = strHeader & ShowElement("Created by      : ", arrPrevious(4), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Last Updated    : ", strNow, vbCrLf)
                    strHeader = strHeader & ShowElement("Updated by      : ", strAuthorName, vbCrLf & vbCrLf)
            End Select
            
            Select Case strFileExt
                Case "asp", "aspx", "ascx", "asmx", "asa", "asax"
                    strHeader = strHeader & ShowElement("QueryString Collection  : ", GetElements(1,"(\.querystring|request|request.item|GetRequestValue)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Form Collection         : ", GetElements(1,"(\.form|GetRequestValue)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Application Variables   : ", GetElements(1,"(application|\.AppSettings)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Session Variables       : ", GetElements(1,"session",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Database Tables         : ", GetElements(3,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Stored Procedures       : ", GetElements(4,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Include Files           : ", GetElements(2,"",strFileContents), vbCrLf)
                    If IsMatch(strHeader, "(QueryString Collection  :|Form Collection         :|Application Variables   :|Session Variables       :|Database Tables         :|Stored Procedures       :|Include Files           :)") Then strHeader = strHeader & vbCrLf End If
                    If strFileExt = "asp" Then
                        strHeader = strHeader & ShowElement("JS Functions            : ", GetElements(9,"",strFileContents), vbCrLf)
                        strHeader = strHeader & ShowElement("VBS Functions           : ", GetElements(6,"",strFileContents), vbCrLf)
                    End if
                    strHeader = strHeader & ShowElement("VBS Subroutines         : ", GetElements(7,"",strFileContents), vbCrLf)
                    strHeader = strHeader & strLineStart & "Comments                : "
                Case "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                    strHeader = strHeader & ShowElement("QueryString Collection  : ", GetElements(5,"(get)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Form Collection         : ", GetElements(5,"(post)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Application Variables   : ", GetElements(5,"application",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Session Variables       : ", GetElements(5,"session",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Database Tables         : ", GetElements(3,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Stored Procedures       : ", GetElements(4,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Include Files           : ", GetElements(1,"(include|require|include_once|require_once)",strFileContents), vbCrLf)
                    If IsMatch(strHeader, "(QueryString Collection  :|Form Collection         :|Application Variables   :|Session Variables       :|Database Tables         :|Stored Procedures       :|Include Files           :)") Then strHeader = strHeader & vbCrLf End If
                    strHeader = strHeader & ShowElement("Functions               : ", GetElements(9,"",strFileContents), vbCrLf)
                    strHeader = strHeader & strLineStart & "Comments                : "
                Case Else
                    strHeader = strHeader & ShowElement("JS Functions    : ", GetElements(9,"",strFileContents), vbCrLf)
                    If strFileExt <> "js" Then
                        strHeader = strHeader & ShowElement("VBS Functions   : ", GetElements(6,"",strFileContents), vbCrLf)
                        strHeader = strHeader & ShowElement("VBS Subroutines : ", GetElements(7,"",strFileContents), vbCrLf)
                    End if
                    strHeader = strHeader & strLineStart & "Comments        : "
            End Select
            
            ' update the header
            objEditor.Text Replace(strFileContents, objMatches.Item(0), strHeader)
            
            ' restore cursor position
            objEditor.setCaretPos intXPos, intYPos
            
        Else
            
            Select Case strFileExt
                Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                    strHeader = strHeader & strHeaderStart
                    strHeader = strHeader & ShowElement("Filename                : ", strFileName, vbCrLf)
                    strHeader = strHeader & ShowElement("URL                     : ", HTTP_Path(strFilePath), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Created                 : ", strCreated, vbCrLf)
                    strHeader = strHeader & ShowElement("Created by              : ", strCreatedBy, vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Last Updated            : ", strNow, vbCrLf)
                    strHeader = strHeader & ShowElement("Updated by              : ", strAuthorName, vbCrLf & vbCrLf)
                Case Else
                    strHeader = strHeader & strHeaderStart
                    strHeader = strHeader & ShowElement("Filename        : ", strFileName, vbCrLf)
                    strHeader = strHeader & ShowElement("URL             : ", HTTP_Path(strFilePath), vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Created         : ", strCreated, vbCrLf)
                    strHeader = strHeader & ShowElement("Created by      : ", strCreatedBy, vbCrLf & vbCrLf)
                    strHeader = strHeader & ShowElement("Last Updated    : ", strNow, vbCrLf)
                    strHeader = strHeader & ShowElement("Updated by      : ", strAuthorName, vbCrLf & vbCrLf)
            End Select
            
            Select Case strFileExt
                Case "asp", "aspx", "ascx", "asmx", "asa", "asax"
                    strHeader = strHeader & ShowElement("QueryString Collection  : ", GetElements(1,"(\.querystring|request|request.item|GetRequestValue)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Form Collection         : ", GetElements(1,"(\.form|GetRequestValue)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Application Variables   : ", GetElements(1,"(application|\.AppSettings)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Session Variables       : ", GetElements(1,"session",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Database Tables         : ", GetElements(3,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Stored Procedures       : ", GetElements(4,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Include Files           : ", GetElements(2,"",strFileContents), vbCrLf)
                    If IsMatch(strHeader, "(QueryString Collection  :|Form Collection         :|Application Variables   :|Session Variables       :|Database Tables         :|Stored Procedures       :|Include Files           :)") Then strHeader = strHeader & vbCrLf End If
                    strHeader = strHeader & ShowElement("JS Functions            : ", GetElements(9,"",strFileContents), vbCrLf)
                    If strFileExt = "asp" Then
                        strHeader = strHeader & ShowElement("VBS Functions           : ", GetElements(6,"",strFileContents), vbCrLf)
                        strHeader = strHeader & ShowElement("VBS Subroutines         : ", GetElements(7,"",strFileContents), vbCrLf)
                    End if
                    strHeader = strHeader & strLineStart & "Comments                : " & vbCrLf
                    strHeader = strHeader & strHeaderEnd
                Case "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                    strHeader = strHeader & ShowElement("QueryString Collection  : ", GetElements(5,"(get)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Form Collection         : ", GetElements(5,"(post)",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Application Variables   : ", GetElements(5,"application",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Session Variables       : ", GetElements(5,"session",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Database Tables         : ", GetElements(3,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Stored Procedures       : ", GetElements(4,"",strFileContents), vbCrLf)
                    strHeader = strHeader & ShowElement("Include Files           : ", GetElements(1,"(include|require|include_once|require_once)",strFileContents), vbCrLf)
                    If IsMatch(strHeader, "(QueryString Collection  :|Form Collection         :|Application Variables   :|Session Variables       :|Database Tables         :|Stored Procedures       :|Include Files           :)") Then strHeader = strHeader & vbCrLf End If
                    strHeader = strHeader & ShowElement("Functions               : ", GetElements(9,"",strFileContents), vbCrLf)
                    strHeader = strHeader & strLineStart & "Comments                : " & vbCrLf
                    strHeader = strHeader & strHeaderEnd
                Case Else
                    strHeader = strHeader & ShowElement("JS Functions    : ", GetElements(9,"",strFileContents), vbCrLf)
                    If strFileExt <> "js" Then
                        strHeader = strHeader & ShowElement("VBS Functions   : ", GetElements(6,"",strFileContents), vbCrLf)
                        strHeader = strHeader & ShowElement("VBS Subroutines : ", GetElements(7,"",strFileContents), vbCrLf)
                    End if
                    strHeader = strHeader & strLineStart & "Comments        : " & vbCrLf
                    strHeader = strHeader & strHeaderEnd
            End Select
            
            ' insert header
            objEditor.lineText strHeader & objEditor.lineText
            
            ' restore cursor position (add the extra lines for the header)
            objEditor.setCaretPos intXPos, intYPos + intExtraLines
            
        End If
        
        objEditor.saveFile()
        
        Set objMatches = Nothing
        Set objRegExp = Nothing
        Set objEditor = Nothing
        
        ' this is a quick fix in order to grab the correct line numbers for 
        ' functions and subs after the extra lines of the header being inserted
        If blnFirstInsert Then Call DoInsert(False) End If
        
    End Sub
    
    Function GetElements(intType, strSearch, strFileContents)
        
        Dim objRegExp, objMatches, match
        Dim strReturn, strComment, strRoutineType, strCommentSymbol
        Dim strCommentLine
        Dim delimPos, i, intFirstIndex, intLastIndex, intCounter
        Dim arrComments
        
        strReturn = ""
        strComment = ""
        intCounter = 0
        
        Select Case intType
            Case 2  ' ---------- SERVER SIDE INCLUDES --------------------------
                strSearch = "<!-- *# *include +(file|virtual) *= *""?(.+)""? *-->"
            Case 3  ' ---------- TABLE NAMES -----------------------------------
                strSearch = " \[?" & DB_TABLE_PREFIX & "\w+\]?\b"
            Case 4 ' ---------- STORED PROCEDURES ------------------------------
                strSearch = STORED_PROCEDURE_PREFIX & "\w+\b"
            Case 5  ' ---------- PHP -------------------------------------------
                strSearch = "\$_" & strSearch & "\[""([^""\s]*)""\]"
            Case 6  ' ---------- VBS FUNCTIONS ---------------------------------
                ' remember arguments
                ' strSearch = "(\n[^\w'\/]*)(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*function {1}([\w_]+ *\(?[^\{\n\)]*\)?)\s*(as\s+\w+\s*)*\n[^\{]"
                
                ' don't remember arguments
                strSearch = "(\n[^\w'\/]*)(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*function {1}([\w_]+)\s*\(?[^\{\n\)]*\)?\s*(as\s+\w+\s*)*\s+[^\{\s]"
                
                ' get comments
'                 strSearch = "('+ *[^\n>%<]+\n)*\n[^\w'\/]*(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*function {1}([\w_]+) *\(?[^\{\n\)]*\)?\s*(as\s+\w+\s*)*\n[^\{]"
            Case 7  ' ---------- VBS SUBROUTINES -------------------------------
                ' remember arguments
                ' strSearch = "(\n[^\w']*)(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*sub {1}([\w_]+ *\(?[^\{\n\)]*\)?)"
                
                ' don't remember arguments
                strSearch = "(\n[^\w']*)(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*sub {1}([\w_]+) *\(?[^\{\n\)]*\)?"
                
                ' get comments
'                 strSearch = "('+ *[^\n>%<]+\n)*\n[^\w']*(Public|Private|Protected|Friend|Protected Friend)*\s*(MustInherit|MustOverride|NotOverridable|Overloads|Overridable|Overrides|Shadows|Shared)*\s*sub {1}([\w_]+) *\(?[^\{\n\)]*\)?"
            Case 8  ' ---------- EXTENAL CSS FILES -----------------------------
                strSearch = "<[\s]*link[\s]+[^>]*href[\s]*=[\s]*(\\*""*|'*)([^\n""]*.css)\1[^>]*>"
            Case 9  ' ---------- JS FUNCTIONS ----------------------------------
                ' remember arguments
                ' strSearch = "(\s)*(function)\s+([\w_]+ *\(.*\))[\s]*\{"
                
                ' don't remember arguments
                strSearch = "(\s)*(function)\s+([\w_]+)\s*\(.*\)\s*\{\s*"
                
                ' get comments
                strSearch = "(\/\/+[\s\S]+|\/+\*+[^(\*\/)]\*\/)*\s*(function)\s+([\w_]+)\s*\(?.*\)?\s*\{"
                strSearch = "(\/\/+.+\n\s*|\/\*+[^(*\/)]+\*+\/\n*\s*)*\s*(function)\s+([\w_]+)\s*\(.*\)\s*\{"
            Case Else   ' ---------- REQUEST COLLECTIONS, SESSION VARIABLES, PHP INCLUDE FILES
                strSearch = strSearch & "\s*(\[|\()""([^""\s]*)""(\]|\))"
        End Select
        
        Set objRegExp = New RegExp
        With objRegExp
            .Global = True
            .IgnoreCase = True
            If intType = 3 OR intType = 4 Then
                .IgnoreCase = False
            End If
            .Pattern = strSearch
            Set objMatches = .Execute(strFileContents)
        End With
        
        If objMatches.Count > 0 Then
            For Each match In objMatches
                If NOT match = "" Then
                
                    intFirstIndex = match.FirstIndex
                    
                    If NOT (intType = 3 OR intType = 4 OR intType = 6 OR intType = 7 OR intType = 9) Then    ' table names / stored procedures
                        If NOT InStr(match, """") > 0 Then
                            match = Left(match, InStrRev(match, "-->") - 1)
                            match = Trim(Mid(match, InStr(match, "=") + 1))
                        Else
                            match = Left(match, InStrRev(match, """") - 1)
                            match = Mid(match, InStr(match, """") + 1)
                        End If
                    ElseIf intType = 3 Then    ' table names
                        match = Mid(match, 2)
                        match = Replace(match, "[", "")
                        match = Replace(match, "]", "")
                    ElseIf intType = 6 OR intType = 7 OR intType = 9 Then    ' functions, subroutines
                        
                        strComment = objRegExp.Replace(match,"$1")
                        
                        If Len(strComment) <> 0 Then
                            
                            '---------------------------------------------------
                            ' workaround to grab multiline comments
                            '---------------------------------------------------
                            If intType = 6 OR intType = 7 Then ' vb
                                strRoutineType = objRegExp.Replace(match,"$3")
                                strCommentSymbol = "'"
                            Else
                                strRoutineType = objRegExp.Replace(match,"$2")
                                strCommentSymbol = "//"
                            End If
                            
                            strComment = Left(match, InStrRev(match, strRoutineType)-1)
                            '---------------------------------------------------
                            
                            strComment = SuperTrim(Mid(strComment,3))
                            strComment = Replace(strComment,"*/","")
                            
                            If Instr(strComment, vbCrLf) > 0 Then
                                arrComments = Split(strComment, vbCrLf)
                                For i = 0 To UBound(arrComments)
                                    strCommentLine = SuperTrim(arrComments(i))
                                    strCommentLine = TrimChar(strCommentLine, strCommentSymbol)
                                    arrComments(i) = SuperTrim(strCommentLine)
                                Next
                                strComment = Join(arrComments, " ")
                            End If
                            strComment = vbTab & strComment
                        End if
                        
                        intLastIndex = intFirstIndex + Len(RSuperTrim(match))
                        
                        If intType = 6 OR intType = 7 Then ' vb
                            match = objRegExp.Replace(match,"$4") & "" & GetLineNo(strFileContents, intLastIndex) '& strComment
                        Else
                            strComment = vbTab & TrimChar(strComment, "*")
                            match = objRegExp.Replace(match,"$3") & "" & GetLineNo(strFileContents, intLastIndex) & strComment
                        End If
                    End If
                    
                    If intType = 6 OR intType = 7 OR intType = 9 Then    ' functions, subroutines
                        Select Case strFileExt
                            Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                                strReturn = strReturn & strLineStart & "                          " & match & vbCrLf
                            Case Else
                                strReturn = strReturn & strLineStart & "                  " & match & vbCrLf
                        End Select
                    Else
                        strReturn = strReturn & match & ", "
                    End If
                    
                    intCounter = intCounter + 1
                End If
            Next
            
            If intType = 6 OR intType = 7 OR intType = 9 Then    ' functions, subroutines
                strReturn = Replace(strReturn, " ", "")
                strReturn = Replace(strReturn, "", " ")
                strReturn = AlignColumns(Left(strReturn, Len(strReturn) - 1), " ")
                strReturn = Replace(strReturn, "", " ")
                strReturn = Replace(strReturn, " Line", "    Line")
                ' strip the leading spacers
                Select Case strFileExt
                    Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml"
                        strReturn = Mid(strReturn, Len(strLineStart) + 26)
                    Case Else
                        strReturn = Mid(strReturn, Len(strLineStart) + 18)
                End Select
                
            Else
                strReturn = Left(strReturn, Len(strReturn) - 2)
            End If
            
        End If
        
        Set objMatches = Nothing
        Set objRegExp = Nothing
        
        strReturn = Trim(strReturn)
        
        If NOT (intType = 6 OR intType = 7 OR intType = 9) Then
            strReturn = SortElements(strReturn)
            strReturn = DeleteDuplicates(strReturn)
        End If
        
        GetElements = strReturn
        
    End Function
    
    Function SortElements(str)
        
        Dim arrTemp, intLen, switch, el1, el2, temp, i
        
        arrTemp = Split(str, ", ")
        intLen = UBound(arrTemp)
        
        Do
            switch = 0
            For i = 0 To (intLen - 1)
                el1 = LCase(arrTemp(i))
                el2 = LCase(arrTemp(i + 1))
                If el1 > el2 Then
                    temp = arrTemp(i)
                    arrTemp(i) = arrTemp(i + 1)
                    arrTemp(i + 1) = temp
                    switch = i
                End If
            Next
            intLen = switch
        Loop While switch
        
        SortElements = Join(arrTemp, ", ")
        
    End Function
    
    Function DeleteDuplicates(str)
        Dim arrTemp, intLen, i, j, k
        
        arrTemp = Split(str, ", ")
        intLen = UBound(arrTemp)
        
        For i = 0 To (intLen)
            
            If i > intLen Then Exit For
            
            For j = i + 1 To (intLen)
                
                If j > intLen Then Exit For
                
                If Trim(arrTemp(i)) = Trim(arrTemp(j)) Then
                    
                    For k = j To (intLen - 1)
                        arrTemp(k) = arrTemp(k + 1)
                    Next
                    
                    ReDim Preserve arrTemp(intLen - 1)
                    intLen = UBound(arrTemp)
                    j = j - 1
                    
                End If
                
            Next
            
        Next
        
        DeleteDuplicates = Rejoin(arrTemp, ", ")
        
    End Function
    
    Function Rejoin(arrTemp, strDelimiter)
        Dim strOutput, strElement, strCheck, strComment
        Dim i, intTotalElements, intLineLen
        
        Select Case strFileExt
            Case "asp", "aspx", "ascx", "asmx", "asa", "asax"
                strComment = "'"
            Case Else
                strComment = ""
        End Select
        
        intTotalElements = UBound(arrTemp)
        
        If intTotalElements >= 0 Then
            intLineLen = Len(arrTemp(0)) + 2
        Else
            intLineLen = 0
        End If
        
        ' loop through collected field names and concatenate output strings
        For i = 0 To intTotalElements
            strElement = arrTemp(i)
            
            If i <> intTotalElements Then
                intLineLen = intLineLen + Len(arrTemp(i + 1)) + 2
            End If
            
            strOutput = strOutput & strElement & ", "
            
            ' after too many elements on a line, insert a line break
            If intLineLen > 52 Then
                strOutput = Left(strOutput, Len(strOutput) - 1)
                strOutput = strOutput & vbCrLf & strComment & "                             "
                intLineLen = Len(strElement) + 2
            End If
        Next
        
        ' trim trailing comma
        If Len(strOutput) <> 0 Then
            strOutput = Left(strOutput, Len(strOutput) - 2)
        End If
        
        Rejoin = strOutput
    End Function
    
    Function HTTP_Path(strFilePath)
        
        Dim i, strHttpPath
        
        Select Case strFileExt
            Case "asp", "aspx", "ascx", "asmx", "asa", "asax", "php", "php3", "php4", "php5", "ph3", "ph4", "phtm", "phtml", "htm", "html", "xhtml", "shtml"
                
                strHttpPath = "http://" & Replace(strFilePath, "\", "/")
                strHttpPath = Replace(strHttpPath, "////", "//")
                strHttpPath = ReplaceExp(strHttpPath, "c:\/inetpub\/wwwroot", "localhost")
                
                For i = 0 To UBound(arrHTTP_Paths, 2)
                    strHttpPath = ReplaceExp(strHttpPath, EscapeRegExpChar(arrHTTP_Paths(INCORRECT, i)), arrHTTP_Paths(CORRECT, i))
                Next
                
            Case Else
                strHttpPath = strFilePath
                For i = 0 To UBound(arrLocal_Paths, 2)
                    strHttpPath = ReplaceExp(strHttpPath, EscapeRegExpChar(arrLocal_Paths(INCORRECT, i)), arrLocal_Paths(CORRECT, i))
                Next
        End Select
        
        If IsMatch(strHttpPath,"http:\/\/\w:\/.+") Then strHttpPath = Replace(strHttpPath, "http://", "") End If
        
        HTTP_Path = Trim(strHttpPath)
        
    End Function
    
    Function ReplaceExp(strInput, strPattern, strReplace)
        Dim objRegExp, strReturn
        
        ReplaceExp = ""
        
        Set objRegExp = New RegExp 
            With objRegExp
                .IgnoreCase = True
                .Global = True
                .Pattern = strPattern
                strReturn = .Replace(strInput, strReplace)
            End With
        Set objRegExp = Nothing
        
        ReplaceExp = strReturn
    End Function
    
    Function EscapeRegExpChar(strInput)
        Dim strReturn
        
        strReturn = Replace(strInput, "\", "\\")
        strReturn = Replace(strReturn, ".", "\.")
        strReturn = Replace(strReturn, "/", "\/")
        strReturn = Replace(strReturn, "(", "\(")
        strReturn = Replace(strReturn, ")", "\)")
        strReturn = Replace(strReturn, "|", "\|")
        strReturn = Replace(strReturn, "[", "\[")
        strReturn = Replace(strReturn, "]", "\]")
        strReturn = Replace(strReturn, "{", "\{")
        strReturn = Replace(strReturn, "}", "\}")
        strReturn = Replace(strReturn, "^", "\^")
        strReturn = Replace(strReturn, "$", "\$")
        strReturn = Replace(strReturn, "*", "\*")
        strReturn = Replace(strReturn, "?", "\?")
        strReturn = Replace(strReturn, "+", "\+")
        
        EscapeRegExpChar = strReturn
    End Function
    
    Function GetLineNo(strFileContents, intLastIndex)
        Dim objRegExp, objMatches, intLineNo
        
        Set objRegExp = New RegExp
            With objRegExp
                .Global = True
                .IgnoreCase = True
                .Pattern = Chr(10)
                Set objMatches = .Execute(Left(strFileContents, intLastIndex))
                    intLineNo = objMatches.Count + 1
                Set objMatches = Nothing
            End With
        Set objRegExp = Nothing
    
        GetLineNo = "Line " & intLineNo
    End Function
    
'     Function GetLineNo(strFileContents, strSearchText)
'         Dim intLineNo, intTotalLines
'         Dim arrFileContents
'         
'         arrFileContents = Split(strFileContents, vbCrLf)
'         intTotalLines = UBound(arrFileContents)
'         
'         For intLineNo = 0 To intTotalLines
'             If InStr(arrFileContents(), strSearchText) > 0 Then
'                 Exit For
'             End If
'         Next
'     
'         GetLineNo = "Line " & intLineNo + 1
'     End Function
    
    ' an adaption of Scott Greenberg's procedure
    Function AlignColumns (text, delimiter)
        Dim Columns(100)
        Dim i, lines, regSearch, regReplace, Matches, Match
        Dim Column, Length, Line, newLine, EOL
        Dim foundText, extraText, Spaces, safeDelimiter
        
        ' Determine end-of-line
        EOL = ""
        If InStr(text, Chr(13)) Then
            EOL = EOL & Chr(13)
        End If
        If InStr(text, Chr(10)) Then
            EOL = EOL & Chr(10)
        End If
        
        ' Get lines
        lines = Split(text, EOL)
        
        ' Make delimiter regexp safe
        Set regReplace = New RegExp
        With regReplace
            .Global     = True
            .IgnoreCase = True
            .Pattern    = "([\$|\(|\)|\*|\+|\.|\[|\]|\?|\\|\^|\{|\}|\|])"
        End With
        safeDelimiter = regReplace.Replace(delimiter, "\$1")
        
        ' Setup regexp object
        Set regSearch = New RegExp
        With regSearch
            .Global     = True
            .IgnoreCase = True
            .Pattern    = "([^" & safeDelimiter & "]*)(\s*)" & safeDelimiter
        End With
        
        ' Get all column's max width
        For Each Line in lines
            Set Matches = regSearch.Execute(Line)
            ' Initialize column index
            Column = 0
            For Each Match in Matches
                Length = len(Match.SubMatches(0))
                If Length > Columns(Column) Then
                    Columns(Column) = Length
                End If
                Column = Column + 1
            Next
        Next
        
        ' Initialize line index
        i = -1
        
        ' Added spacing
        For Each Line in lines
            Column = 0
            newLine = ""
            i = i + 1
            Set Matches = regSearch.Execute(Line)
            
            ' Sum matches and remove from orignal line
            ' Match should catch everything before delimiter
            foundText = ""
            For Each Match in Matches
                foundText = foundText & Match.SubMatches(0) & Match.SubMatches(1) & delimiter
            Next
            extraText = Replace(Line, foundText, "")
            
            For Each Match in Matches
                Spaces  = Columns(Column) - Len(Match.SubMatches(0))
                newLine = newLine & Match.SubMatches(0) & String(Spaces, " ") & delimiter
                Column  = Column + 1
            Next
            If newLine <> "" Then
                lines(i) = newLine & extraText
            End If
        Next
        
        AlignColumns = Join(lines, vbCrLf)
    End Function
    
    Function ShowElement(strCaption, strElement, strLineBreak)
        strElement = Trim(strElement)
        If Len(strElement) Then
            ShowElement = strLineStart & strCaption & strElement & strLineBreak
        Else
            ShowElement = ""
        End If
    End Function
    
    ' returns a boolean value indicating whether a regular expression patten exists in a string
    Private Function IsMatch(sInput, sPattern)
        Dim oRegEx
        
        Set oRegEx = New RegExp
            With oRegEx
                .Global = True
                .IgnoreCase = True
                .Pattern = sPattern
                IsMatch = oRegEx.Test(sInput)
            End With
        Set oRegEx = Nothing
    
    End Function
    
    Function RSuperTrim(ByVal s)
        If IsNull(s) Then
            SuperTrim = s
        Else
            s = Trim(s)
            Do While Len(s) > 0
                If Asc(Right(s, 1)) < 33 Then
                    s = Left(s, Len(s) - 1)
                Else
                    Exit Do
                End If
            Loop
            RSuperTrim = s
        End If
    End Function
    
    Function SuperTrim(ByVal s)
        If IsNull(s) Then
            SuperTrim = s
        Else
            s = Trim(s)
            Do While Len(s) > 0
                If Asc(Left(s, 1)) < 33 Then
                    s = Mid(s, 2)
                ElseIf Asc(Right(s, 1)) < 33 Then
                    s = Left(s, Len(s) - 1)
                Else
                    Exit Do
                End If
            Loop
            SuperTrim = s
        End If
    End Function
    
    Function TrimChar(ByVal strInput, strTrimChar)
        Dim intTrimCharLen
        If IsNull(strInput) OR IsNull(strTrimChar) Then
            TrimChar = strInput
        Else
            strInput = SuperTrim(strInput)
            intTrimCharLen = Len(strTrimChar)
            Do While Len(strInput) > 0
                If Left(strInput, intTrimCharLen) = strTrimChar Then
                    strInput = Mid(strInput, intTrimCharLen + 1)
                ElseIf Right(strInput, intTrimCharLen) = strTrimChar Then
                    strInput = Left(strInput, Len(strInput) - intTrimCharLen)
                Else
                    Exit Do
                End If
            Loop
            TrimChar = Trim(strInput)
        End If
    End Function
